@@ -2,18 +2,22 @@
 from __future__ import print_function
 
 import threading
-# Python3 queue support.
-try:
-    import Queue
-except ImportError:
-    import queue as Queue
 import time
+import re
+import sys
+import six
 
 import logging
-logging.basicConfig()
-logger = logging.getLogger('Telebot')
-import re
-from telebot import apihelper, types
+logger = logging.getLogger('TeleBot')
+formatter = logging.Formatter('%(asctime)s (%(filename)s:%(lineno)d) %(levelname)s - %(name)s: "%(message)s"')
+
+console_output_handler = logging.StreamHandler(sys.stderr)
+console_output_handler.setFormatter(formatter)
+logger.addHandler(console_output_handler)
+
+logger.setLevel(logging.ERROR)
+
+from telebot import apihelper, types, util
 
 """
 Module : telebot
@@ -22,47 +26,6 @@ Module : telebot
 API_URL = r"https://api.telegram.org/"
 
 
-class ThreadPool:
-    class WorkerThread(threading.Thread):
-        count = 0
-
-        def __init__(self, queue):
-            threading.Thread.__init__(self, name="WorkerThread{0}".format(self.__class__.count + 1))
-            self.__class__.count += 1
-            self.queue = queue
-            self.daemon = True
-
-            self._running = True
-            self.start()
-
-        def run(self):
-            while self._running:
-                try:
-                    task, args, kwargs = self.queue.get()
-                    task(*args, **kwargs)
-                except Queue.Empty:
-                    time.sleep(0)
-                    pass
-
-        def stop(self):
-            self._running = False
-
-    def __init__(self, num_threads=4):
-        self.tasks = Queue.Queue()
-        self.workers = [self.WorkerThread(self.tasks) for _ in range(num_threads)]
-
-        self.num_threads = num_threads
-
-    def put(self, func, *args, **kwargs):
-        self.tasks.put((func, args, kwargs))
-
-    def close(self):
-        for worker in self.workers:
-            worker.stop()
-        for worker in self.workers:
-            worker.join()
-
-
 class TeleBot:
     """ This is TeleBot Class
     Methods:
@@ -82,7 +45,6 @@ class TeleBot:
 
     def __init__(self, token, create_threads=True, num_threads=4):
         """
-
         :param token: bot API token
         :param create_threads: Create thread for message handler
         :param num_threads: Number of worker in thread pool.
@@ -104,7 +66,21 @@ class TeleBot:
 
         self.message_handlers = []
         if self.__create_threads:
-            self.worker_pool = ThreadPool(num_threads)
+            self.worker_pool = util.ThreadPool(num_threads)
+
+    def get_updates(self, offset=None, limit=None, timeout=20):
+        """
+        Use this method to receive incoming updates using long polling (wiki). An Array of Update objects is returned.
+        :param offset: Integer. Identifier of the first update to be returned.
+        :param limit: Integer. Limits the number of updates to be retrieved.
+        :param timeout: Integer. Timeout in seconds for long polling.
+        :return: array of Updates
+        """
+        json_updates = apihelper.get_updates(self.token, offset, limit, timeout)
+        ret = []
+        for ju in json_updates:
+            ret.append(types.Update.de_json(ju))
+        return ret
 
     def get_update(self):
         """
@@ -112,14 +88,13 @@ class TeleBot:
         Registered listeners and applicable message handlers will be notified when a new message arrives.
         :raises ApiException when a call has failed.
         """
-        updates = apihelper.get_updates(self.token, offset=(self.last_update_id + 1), timeout=20)
+        updates = self.get_updates(offset=(self.last_update_id + 1), timeout=3)
         new_messages = []
         for update in updates:
-            if update['update_id'] > self.last_update_id:
-                self.last_update_id = update['update_id']
-            msg = types.Message.de_json(update['message'])
-            new_messages.append(msg)
-        logger.debug('GET %d new messages' % len(new_messages))
+            if update.update_id > self.last_update_id:
+                self.last_update_id = update.update_id
+            new_messages.append(update.message)
+        logger.debug('Received {0} new messages'.format(len(new_messages)))
         if len(new_messages) > 0:
             self.process_new_messages(new_messages)
 
@@ -136,7 +111,7 @@ class TeleBot:
             else:
                 listener(new_messages)
 
-    def polling(self, none_stop=False, interval=0):
+    def polling(self, none_stop=False, interval=0, block=True):
         """
         This function creates a new Thread that calls an internal __polling function.
         This allows the bot to retrieve Updates automagically and notify listeners and message handlers accordingly.
@@ -155,18 +130,34 @@ class TeleBot:
         self.polling_thread.daemon = True
         self.polling_thread.start()
 
+        if block:
+            while self.polling_thread.is_alive:
+                try:
+                    time.sleep(.1)
+                except KeyboardInterrupt:
+                    logger.info("Received KeyboardInterrupt. Stopping.")
+                    self.stop_polling()
+                    self.polling_thread.join()
+                    break
+
     def __polling(self, none_stop, interval):
-        logger.info('TeleBot: Started polling.')
+        logger.info('Started polling.')
+
+        error_interval = .25
         while not self.__stop_polling.wait(interval):
             try:
                 self.get_update()
-            except Exception as e:
+                error_interval = .25
+            except apihelper.ApiException as e:
                 if not none_stop:
                     self.__stop_polling.set()
-                    logger.info("TeleBot: Exception occurred. Stopping.")
+                    logger.info("Exception occurred. Stopping.")
+                else:
+                    time.sleep(error_interval)
+                    error_interval *= 2
                 logger.error(e)
 
-        logger.info('TeleBot: Stopped polling.')
+        logger.info('Stopped polling.')
 
     def stop_polling(self):
         self.__stop_polling.set()
@@ -190,7 +181,8 @@ class TeleBot:
         result = apihelper.get_user_profile_photos(self.token, user_id, offset, limit)
         return types.UserProfilePhotos.de_json(result)
 
-    def send_message(self, chat_id, text, disable_web_page_preview=None, reply_to_message_id=None, reply_markup=None):
+    def send_message(self, chat_id, text, disable_web_page_preview=None, reply_to_message_id=None, reply_markup=None,
+                     parse_mode=None):
         """
         Use this method to send text messages.
 
@@ -202,11 +194,12 @@ class TeleBot:
         :param disable_web_page_preview:
         :param reply_to_message_id:
         :param reply_markup:
+        :param parse_mode:
         :return: API reply.
         """
         return types.Message.de_json(
             apihelper.send_message(self.token, chat_id, text, disable_web_page_preview, reply_to_message_id,
-                                   reply_markup))
+                                   reply_markup, parse_mode))
 
     def forward_message(self, chat_id, from_chat_id, message_id):
         """
@@ -231,7 +224,8 @@ class TeleBot:
         return types.Message.de_json(
             apihelper.send_photo(self.token, chat_id, photo, caption, reply_to_message_id, reply_markup))
 
-    def send_audio(self, chat_id, audio, duration=None, performer=None, title=None, reply_to_message_id=None, reply_markup=None):
+    def send_audio(self, chat_id, audio, duration=None, performer=None, title=None, reply_to_message_id=None,
+                   reply_markup=None):
         """
         Use this method to send audio files, if you want Telegram clients to display them in the music player. Your audio must be in the .mp3 format.
         :param chat_id:Unique identifier for the message recipient
@@ -239,12 +233,13 @@ class TeleBot:
         :param duration:Duration of the audio in seconds
         :param performer:Performer
         :param title:Track name
-        :param reply_to_message_id:If the message is a reply, ID of the original messag
+        :param reply_to_message_id:If the message is a reply, ID of the original message
         :param reply_markup:
         :return: Message
         """
         return types.Message.de_json(
-            apihelper.send_audio(self.token, chat_id, audio,duration,performer,title, reply_to_message_id, reply_markup))
+            apihelper.send_audio(self.token, chat_id, audio, duration, performer, title, reply_to_message_id,
+                                 reply_markup))
 
     def send_voice(self, chat_id, voice, duration=None, reply_to_message_id=None, reply_markup=None):
         """
@@ -257,7 +252,7 @@ class TeleBot:
         :return: Message
         """
         return types.Message.de_json(
-            apihelper.send_voice(self.token, chat_id, voice, duration, reply_to_message_id,reply_markup))
+            apihelper.send_voice(self.token, chat_id, voice, duration, reply_to_message_id, reply_markup))
 
     def send_document(self, chat_id, data, reply_to_message_id=None, reply_markup=None):
         """
@@ -409,31 +404,38 @@ class TeleBot:
         :param func: Optional lambda function. The lambda receives the message to test as the first parameter. It must return True if the command should handle the message.
         :param content_types: This commands' supported content types. Must be a list. Defaults to ['text'].
         """
-
         def decorator(fn):
-            func_dict = {'function': fn, 'content_types': content_types}
+            handler_dict = {'function': fn}
+            filters = {'content_types': content_types}
             if regexp:
-                func_dict['regexp'] = regexp if 'text' in content_types else None
+                filters['regexp'] = regexp
             if func:
-                func_dict['lambda'] = func
+                filters['lambda'] = func
             if commands:
-                func_dict['commands'] = commands if 'text' in content_types else None
-            self.message_handlers.append(func_dict)
+                filters['commands'] = commands
+            handler_dict['filters'] = filters
+            self.message_handlers.append(handler_dict)
             return fn
 
         return decorator
 
     @staticmethod
     def _test_message_handler(message_handler, message):
-        if message.content_type not in message_handler['content_types']:
-            return False
-        if 'commands' in message_handler and message.content_type == 'text':
-            return apihelper.extract_command(message.text) in message_handler['commands']
-        if 'regexp' in message_handler and message.content_type == 'text' and re.search(message_handler['regexp'],
-                                                                                        message.text):
-            return True
-        if 'lambda' in message_handler:
-            return message_handler['lambda'](message)
+        for filter, filter_value in six.iteritems(message_handler['filters']):
+            if not TeleBot._test_filter(filter, filter_value, message):
+                return False
+        return True
+
+    @staticmethod
+    def _test_filter(filter, filter_value, message):
+        if filter == 'content_types':
+            return message.content_type in filter_value
+        if filter == 'regexp':
+            return message.content_type == 'text' and re.search(filter_value, message.text)
+        if filter == 'commands':
+            return message.content_type == 'text' and util.extract_command(message.text) in filter_value
+        if filter == 'func':
+            return filter_value(message)
         return False
 
     def _notify_command_handlers(self, new_messages):
@@ -442,93 +444,55 @@ class TeleBot:
                 if self._test_message_handler(message_handler, message):
                     if self.__create_threads:
                         self.worker_pool.put(message_handler['function'], message)
-                        # t = threading.Thread(target=message_handler['function'], args=(message,))
-                        # t.start()
                     else:
                         message_handler['function'](message)
                     break
 
 
-class AsyncTask:
-    def __init__(self, target, *args, **kwargs):
-        self.target = target
-        self.args = args
-        self.kwargs = kwargs
-
-        self.done = False
-        self.thread = threading.Thread(target=self._run)
-        self.thread.start()
-
-    def _run(self):
-        try:
-            self.result = self.target(*self.args, **self.kwargs)
-        except Exception as e:
-            self.result = e
-        self.done = True
-
-    def wait(self):
-        if not self.done:
-            self.thread.join()
-        if isinstance(self.result, Exception):
-            raise self.result
-        else:
-            return self.result
-
-
-def async():
-    def decorator(fn):
-        def wrapper(*args, **kwargs):
-            return AsyncTask(fn, *args, **kwargs)
-
-        return wrapper
-
-    return decorator
-
-
 class AsyncTeleBot(TeleBot):
     def __init__(self, *args, **kwargs):
         TeleBot.__init__(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def get_me(self):
         return TeleBot.get_me(self)
 
-    @async()
+    @util.async()
     def get_user_profile_photos(self, *args, **kwargs):
         return TeleBot.get_user_profile_photos(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_message(self, *args, **kwargs):
         return TeleBot.send_message(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def forward_message(self, *args, **kwargs):
         return TeleBot.forward_message(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_photo(self, *args, **kwargs):
         return TeleBot.send_photo(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_audio(self, *args, **kwargs):
         return TeleBot.send_audio(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_document(self, *args, **kwargs):
         return TeleBot.send_document(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_sticker(self, *args, **kwargs):
         return TeleBot.send_sticker(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_video(self, *args, **kwargs):
         return TeleBot.send_video(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_location(self, *args, **kwargs):
         return TeleBot.send_location(self, *args, **kwargs)
 
-    @async()
+    @util.async()
     def send_chat_action(self, *args, **kwargs):
         return TeleBot.send_chat_action(self, *args, **kwargs)
