@@ -28,7 +28,6 @@ class JsonSerializable:
     Subclasses of this class are guaranteed to be able to be converted to JSON format.
     All subclasses of this class must override to_json.
     """
-
     def to_json(self):
         """
         Returns a JSON string representation of this class.
@@ -44,7 +43,6 @@ class JsonDeserializable:
     Subclasses of this class are guaranteed to be able to be created from a json-style dict or json formatted string.
     All subclasses of this class must override de_json.
     """
-
     @classmethod
     def de_json(cls, json_type):
         """
@@ -70,6 +68,27 @@ class JsonDeserializable:
         else:
             raise ValueError("json_type should be a json dict or string.")
 
+    def __str__(self):
+        d = {}
+        for x, y in self.__dict__.iteritems():
+            if hasattr(y, '__dict__'):
+                d[x] = y.__dict__
+            else:
+                d[x] = y
+
+        return unicode(d)
+
+class Update(JsonDeserializable):
+    @classmethod
+    def de_json(cls, json_type):
+        obj = cls.check_json(json_type)
+        update_id = obj['update_id']
+        message = Message.de_json(obj['message'])
+        return Update(update_id, message)
+
+    def __init__(self, update_id, message):
+        self.update_id = update_id
+        self.message = message
 
 class User(JsonDeserializable):
     @classmethod
